/*
 * Copyright (c) 2011-2015 Chris McCormick, Sébastien Piquemal <sebpiq@gmail.com>
 *
 *  This file is part of WebPd. See https://github.com/sebpiq/WebPd for documentation
 *
 *  WebPd is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  WebPd is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with WebPd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var _ = require('underscore')
  , utils = require('./utils')

// Base for outlets and inlets. Mostly handles connections and disconnections
var Portlet = exports.Portlet = function(obj, id) {
  this.obj = obj
  this.id = id
  this.connections = []
  this.init()
}

_.extend(Portlet.prototype, {

/******************** Methods to implement *****************/

  // True if the portlet can connect objects belonging to different patches
  crossPatch: false,

  // This method is called when the portlet is initialized.
  init: function() {},

  // This method is called when the object is started
  start: function() {},

  // This method is called after all objects have been stopped
  stop: function() {},

  // This method is called when the portlet receives a message.
  message: function(args) {},

  // This method is called when the portlet gets a new connection,
  // and when the portlet's object is started it is called again.
  connection: function(otherPortlet) {},

  // This method is called when the portlet gets disconnected.
  disconnection: function(otherPortlet) {},


/************************* Public API **********************/

  // Connects the calling portlet with `otherPortlet` 
  // Returns true if a connection was indeed established.
  connect: function(otherPortlet) {
    if (this.connections.indexOf(otherPortlet) !== -1) return false
    if (!(this.crossPatch || otherPortlet.crossPatch)
    && this.obj.patch !== otherPortlet.obj.patch)
      throw new Error('cannot connect objects that belong to different patches')
    this.connections.push(otherPortlet)
    otherPortlet.connect(this)
    this.connection(otherPortlet)
    return true
  },

  // Generic function for disconnecting the calling portlet 
  // from  `otherPortlet`. Returns true if a disconnection was indeed made
  disconnect: function(otherPortlet) {
    var connInd = this.connections.indexOf(otherPortlet)
    if (connInd === -1) return false
    this.connections.splice(connInd, 1)
    otherPortlet.disconnect(this)
    this.disconnection(otherPortlet)
    return true
  }

})
Portlet.extend = utils.chainExtend

// Base inlet
var Inlet = exports.Inlet = Portlet.extend({})

// Base outlet
var Outlet = exports.Outlet = Portlet.extend({})
